import 'package:analyzer/dart/element/element.dart' as az;
import 'package:analyzer/dart/element/type.dart' as az;
import 'package:build/build.dart';
import 'package:demo/annotation/interceptable.dart';
import 'package:source_gen/source_gen.dart';
import 'package:interceptable/interceptable.dart';
import 'package:renderable/renderable.dart';

class InterceptableGenerator extends GeneratorForAnnotation<Interceptable> {
  @override
  generateForAnnotatedElement(az.Element element, ConstantReader annotation, BuildStep buildStep) {
    ClassElement itClass = InterceptableManager.create(element, buildStep.inputId.uri.toString());

    itClass.fields.take(2).forEach(
          (e) => InterceptorUtil.applyFieldInterceptor(
                e,
                FieldInterceptor(
                  value: RawStatement("null"),
                ),
              ),
        );

    itClass.fields.getRange(2, 4).forEach(
          (e) => InterceptorUtil.applyFieldInterceptor(
                e,
                AccessorInterceptor(
                  getterBodyInterceptor: FunctionBodyInterceptor(
                    after: [
                      RawStatement("value = null;"),
                    ],
                    before: [
                      RawStatement("/// before"),
                    ],
                  ),
                  setterBodyInterceptor: FunctionBodyInterceptor(
                    before: [
                      RawStatement("value = null;"),
                    ],
                    after: [
                      RawStatement("/// after"),
                    ],
                  ),
                ),
              ),
        );

    return FileElement(
      imports: InterceptableManager.importDict[element.library.id],
      renders: [
        itClass,
      ],
    ).render();
  }
}
