import 'package:analyzer/dart/element/element.dart' as az;
import 'package:analyzer/dart/element/type.dart' as az;
import 'package:interceptable/src/element/interceptable_field_element.dart';
import 'package:interceptable/src/element/interceptable_method_element.dart';
import 'package:interceptable/src/misc/interceptable_function_body.dart';
import 'package:interceptable/src/util/parameter_util.dart';
import 'package:renderable/renderable.dart';

class InterceptableElementConverter extends AnalyzerElementConverter {
  InterceptableElementConverter(int id) : super(id);

  @override
  List<ImportElement> parseImports(List<az.ImportElement> imports) {
    return imports.map(parseImport).toList();
  }

  @override
  ClassElement parseClass(az.ClassElement element) {
    if (element == null) return null;

    AnalyzerImportLoopUp.parse(element);

    List<TypeParameterElement> typeParameters =
        element.typeParameters.map(parseTypeParameterElement).toList();

    return ClassElement(
      name: '\$_${element.name}',
      supertype: InterfaceType(
        name: element.name,
        typeArguments:
            typeParameters.map(analyzerTypeConverter.parseTypeArgumentFromTypeParameter).toList(),
      ),
      typeParameters: typeParameters,
      fields: element.fields.map(parseFieldElement).toList(),
      methods: element.methods.map(parseMethodElement).toList(),
    );
  }

  @override
  InterceptableFieldElement parseFieldElement(az.FieldElement field) {
    return InterceptableFieldElement(
      name: field.name,
      type: analyzerTypeConverter.parseDartType(field.type),
      isStatic: field.isStatic,
      isFinal: field.isFinal,
      isConst: field.isConst,
      isSynthetic: field.isSynthetic,
      value: null,
      getter: parsePropertyAccessorElement(field.getter),
      setter: parsePropertyAccessorElement(field.setter),
    );
  }

  PropertyAccessorElement parsePropertyAccessorElement(az.PropertyAccessorElement accessor) {
    if (accessor == null) return null;

    return PropertyAccessorElement(
      name: accessor.displayName,
      isGetter: accessor.isGetter,
      isSetter: accessor.isSetter,
      isStatic: accessor.isSetter,
      parameters: accessor.parameters.map(parseParameter).map((e) => e..name = "value").toList(),
      type: analyzerTypeConverter.parseDartType(accessor.type),
      returnType: analyzerTypeConverter.parseDartType(accessor.returnType),
//      typeParameters: accessor.typeParameters.map(parseTypeParameterElement).toList(),
      body: InterceptableFunctionBody(
        statements: accessor.isSetter
            ? [
                RawStatement("super.${accessor.displayName} = value;"),
              ]
            : [
                VariableDeclarationStatement(
                  type: DartType.var_,
                  name: "value",
                  value: RawStatement("super.${accessor.name}"),
                ),
              ],
        returnStatement: accessor.isGetter
            ? ReturnStatement(
                returnValue: Variable(name: "value"),
              )
            : null,
      ),
    );
  }

  @override
  InterceptableMethodElement parseMethodElement(az.MethodElement method) {
    List<ParameterElement> parameters = method.parameters.map(parseParameter).toList();

    return InterceptableMethodElement(
      isAbstract: method.isAbstract,
      isAsynchronous: method.isAsynchronous,
      isStatic: method.isStatic,
      isExternal: method.isExternal,
      isGenerator: method.isGenerator,
      name: method.name,
      returnType: analyzerTypeConverter.parseDartType(method.returnType),
      typeParameters: method.typeParameters.map(parseTypeParameterElement).toList(),
      parameters: parameters,
      body: InterceptableFunctionBody(
        statements: method.isGenerator
            ? [
                RawStatement(
                  "yield super.${method.name}(${ParameterUtil.generatePassingString(parameters)});",
                ),
              ]
            : [],
        returnStatement: method.isAbstract || method.isGenerator
            ? null
            : ReturnStatement(
                returnValue: RawValue(
                  "super.${method.name}(${ParameterUtil.generatePassingString(parameters)})",
                ),
              ),
      ),
    );
  }
}
